library(stockassessment)

read_matrices_from_file <- function(file_name) {
  data <- readLines(file_name)
  
  # Initialize empty list to store matrices
  matrices <- list()
  current_matrix <- NULL
  matrix_data <- NULL
  
  for (line in data) {
    if (grepl("# Year ", line)) {
      current_matrix <- gsub("# Year ", "", line)
      matrix_data <- NULL
    } else if (line == "# ") {
      matrices[[current_matrix]] <- as.matrix(read.table(text = matrix_data))
    } else {
      matrix_data <- c(matrix_data, line)
    }
  }
  
  return(matrices)
}


setwd("data")  
cnW<-read.ices("cn.dat")
cnWO<-read.ices("cnWithout.dat")
cw<-read.ices("cw.dat")
dw<-read.ices("dw.dat")
lf<-read.ices("lf.dat")
lw<-read.ices("lw.dat")
mo<-read.ices("mo.dat")
nm<-read.ices("nm.dat")
pf<-read.ices("pf.dat")
pm<-read.ices("pm.dat")
sw<-read.ices("sw.dat")
surveys<-read.ices("survey.dat")

covWinterSpatioTemporal <- read_matrices_from_file("covar_cutoff-80.dat")
attr(surveys$`BS-NoRu-Q1_spatioTemporal (BTr)`, "cov-weight") <- covWinterSpatioTemporal


dat<-setup.sam.data(surveys=surveys,
                    residual.fleet=cnWO, 
                    prop.mature=mo, 
                    stock.mean.weight=sw, 
                    catch.mean.weight=cw, 
                    dis.mean.weight=dw, 
                    land.mean.weight=lw,
                    prop.f=pf, 
                    prop.m=pm, 
                    natural.mortality=nm, 
                    land.frac=lf)

save(dat, cnWO, cnW, file="../run/data.RData")
